/* JTZ, removed prefixes for illegal opcodes and
        changed all mnemonics to lower case
*/

/*****************************************************************************

   This file is part of Virtual VCS, the Atari 2600 Emulator
   ===================================================

   Copyright 1996 Alex Hornby. For contributions see the file CREDITS.

   This software is distributed under the terms of the GNU General Public
   License. This is free software with ABSOLUTELY NO WARRANTY.

   See the file COPYING for Details.

   $Id: table.c,v 1.3 1996/03/21 16:36:01 alex Exp $
******************************************************************************/

/*
 * $Id: table.c,v 1.3 1996/03/21 16:36:01 alex Exp $
 *
 * This was part of the x64 Commodore 64 emulator.
 * See README for copyright notice
 *
 * This file contains lookup-table which is used to translate
 * MOS6502 machine instructions. Machine code is used as index
 * to array called lookup. Pointer to function is then fetched
 * from array and function is called.
 * Timing of the undocumented opcodes is based on information
 * in an a"RTI"cle in C=Lehti by Kai Lindfors and Topi Maurola.
 *
 *
 * Written by
 *   Vesa-Matti Puro (vmp@lut.fi)
 *   Jarkko Sonninen (sonninen@lut.fi)
 *   Jouko Valta (jopi@stekt.oulu.fi)
 *
 */

struct lookup_tag {
	char          *mnemonic;	/* Selfdocumenting? */
	short          addr_mode;
	unsigned char  source;
	unsigned char  destination;
	unsigned char  cycles;
	unsigned char  pbc_fix;	/* Cycle for Page Boundary Crossing */
};


/* Addressing mode (addr_mode) is used when instruction is diassembled
 * or assembled by diassembler or assembler. This is used i.e.
 * in function char *sprint_opcode() in the file misc.c.
 *
 * MOS6502 addressing modes are #defined in the file "vmachine.h".
 *
 * Mnemonic is character string telling the name of the instruction.
 */

#define M_NONE	0
#define M_AC 	1
#define M_XR	2
#define M_YR	3
#define M_SP	4
#define M_SR	5
#define M_PC	6
#define M_IMM	7
#define M_ZERO	8
#define M_ZERX	9
#define M_ZERY	10
#define M_ABS	11
#define M_ABSX	12
#define M_ABSY	13
#define M_AIND	14
#define M_INDX	15
#define M_INDY	16
#define M_REL	17
#define M_FC	18
#define M_FD	19
#define M_FI	20
#define M_FV	21
#define M_ADDR	22
#define M_	23

#define M_ACIM	24	/* Source: AC & IMMED (bus collision) */
#define M_ANXR	25	/* Source: AC & XR (bus collision) */
#define M_AXIM	26	/* Source: (AC | #EE) & XR & IMMED (bus collision) */
#define M_ACNC	27	/* Dest: M_AC and Carry = Negative */
#define M_ACXR	28	/* Dest: M_AC, M_XR */

#define M_SABY	29	/* Source: (ABS_Y & SP) (bus collision) */
#define M_ACXS	30	/* Dest: M_AC, M_XR, M_SP */
#define M_STH0	31	/* Dest: Store (src & Addr_Hi+1) to (Addr +0x100) */
#define M_STH1	32
#define M_STH2	33
#define M_STH3	34

#define IMPLIED		0
#define ACCUMULATOR	1
#define IMMEDIATE	2

#define ZERO_PAGE	3
#define ZERO_PAGE_X	4
#define ZERO_PAGE_Y	5

#define ABSOLUTE	6
#define ABSOLUTE_X	7
#define ABSOLUTE_Y	8

#define ABS_INDIRECT	9
#define INDIRECT_X	10
#define INDIRECT_Y	11

#define RELATIVE	12

#define ASS_CODE	13

int     clength[] = {1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 2, 2, 2, 0};

struct lookup_tag lookup[] = {

	/****  Positive  ****/

    /* 00 */ {  "brk",  IMPLIED, M_NONE, M_PC, 7, 0},   /* Pseudo Absolute */
    /* 01 */ {  "ora",  INDIRECT_X, M_INDX, M_AC, 6, 0},    /* (Indirect,X) */
    /* 02 */ {  "jam",  IMPLIED, M_NONE, M_NONE, 0, 0}, /* TILT */
    /* 03 */ {  "slo",  INDIRECT_X, M_INDX, M_INDX, 8, 0},

    /* 04 */ {  "nop",  ZERO_PAGE, M_NONE, M_NONE, 3, 0},
    /* 05 */ {  "ora",  ZERO_PAGE, M_ZERO, M_AC, 3, 0}, /* Zeropage */
    /* 06 */ {  "asl",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},   /* Zeropage */
    /* 07 */ {  "slo",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},

	/* 08 */ {	"php",	IMPLIED, M_SR, M_NONE, 3, 0},
    /* 09 */ {  "ora",  IMMEDIATE, M_IMM, M_AC, 2, 0},  /* Immediate */
    /* 0a */ {  "asl",  ACCUMULATOR, M_AC, M_AC, 2, 0}, /* Accumulator */
    /* 0b */ {  "anc",  IMMEDIATE, M_ACIM, M_ACNC, 2, 0},

    /* 0c */ {  "nop",  ABSOLUTE, M_NONE, M_NONE, 4, 0},
    /* 0d */ {  "ora",  ABSOLUTE, M_ABS, M_AC, 4, 0},   /* Absolute */
    /* 0e */ {  "asl",  ABSOLUTE, M_ABS, M_ABS, 6, 0},  /* Absolute */
    /* 0f */ {  "slo",  ABSOLUTE, M_ABS, M_ABS, 6, 0},

	/* 10 */ {	"bpl",	RELATIVE, M_REL, M_NONE, 2, 0},
    /* 11 */ {  "ora",  INDIRECT_Y, M_INDY, M_AC, 5, 1}, /* (Indirect),Y */
    /* 12 */ {  "jam",  IMPLIED, M_NONE, M_NONE, 0, 0}, /* TILT */
    /* 13 */ {  "slo",  INDIRECT_Y, M_INDY, M_INDY, 8, 0},

    /* 14 */ {  "nop",  ZERO_PAGE_X, M_NONE, M_NONE, 4, 0},
    /* 15 */ {  "ora",  ZERO_PAGE_X, M_ZERX, M_AC, 4, 0},   /* Zeropage,X */
    /* 16 */ {  "asl",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0},/* Zeropage,X */
    /* 17 */ {  "slo",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0},

    /* 18 */ {  "clc",  IMPLIED, M_NONE, M_FC, 2, 0},
    /* 19 */ {  "ora",  ABSOLUTE_Y, M_ABSY, M_AC, 4, 1},    /* Absolute,Y */
    /* 1a */ {  "nop",  IMPLIED, M_NONE, M_NONE, 2, 0},
    /* 1b */ {  "slo",  ABSOLUTE_Y, M_ABSY, M_ABSY, 7, 0},

    /* 1c */ {  "nop",  ABSOLUTE_X, M_NONE, M_NONE, 4, 1},
    /* 1d */ {  "ora",  ABSOLUTE_X, M_ABSX, M_AC,   4, 1},  /* Absolute,X */
    /* 1e */ {  "asl",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0},  /* Absolute,X */
    /* 1f */ {  "slo",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0},

	/* 20 */ {	"jsr",	ABSOLUTE, M_ADDR, M_PC, 6, 0},
    /* 21 */ {  "and",  INDIRECT_X, M_INDX, M_AC, 6, 0},    /* (Indirect ,X) */
    /* 22 */ {  "jam",  IMPLIED, M_NONE, M_NONE,    0, 0},  /* TILT */
    /* 23 */ {  "rla",  INDIRECT_X, M_INDX, M_INDX, 8, 0},

    /* 24 */ {  "bit",  ZERO_PAGE, M_ZERO, M_NONE, 3, 0},   /* Zeropage */
    /* 25 */ {  "and",  ZERO_PAGE, M_ZERO, M_AC,   3, 0},   /* Zeropage */
    /* 26 */ {  "rol",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},   /* Zeropage */
    /* 27 */ {  "rla",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},

	/* 28 */ {	"plp",	IMPLIED, M_NONE, M_SR, 4, 0},
    /* 29 */ {  "and",  IMMEDIATE, M_IMM, M_AC, 2, 0},  /* Immediate */
    /* 2a */ {  "rol",  ACCUMULATOR, M_AC, M_AC, 2, 0}, /* Accumulator */
    /* 2b */ {  "anc",  IMMEDIATE, M_ACIM, M_ACNC, 2, 0},

    /* 2c */ {  "bit",  ABSOLUTE, M_ABS, M_NONE, 4, 0}, /* Absolute */
    /* 2d */ {  "and",  ABSOLUTE, M_ABS, M_AC,  4, 0},  /* Absolute */
    /* 2e */ {  "rol",  ABSOLUTE, M_ABS, M_ABS, 6, 0},  /* Absolute */
    /* 2f */ {  "rla",  ABSOLUTE, M_ABS, M_ABS, 6, 0},

	/* 30 */ {	"bmi",	RELATIVE, M_REL, M_NONE, 2, 0},
    /* 31 */ {  "and",  INDIRECT_Y, M_INDY, M_AC, 5, 1},    /* (Indirect),Y */
    /* 32 */ {  "jam",  IMPLIED, M_NONE, M_NONE, 0, 0}, /* TILT */
    /* 33 */ {  "rla",  INDIRECT_Y, M_INDY, M_INDY, 8, 0},

    /* 34 */ {  "nop",  ZERO_PAGE_X, M_NONE, M_NONE, 4, 0},
    /* 35 */ {  "and",  ZERO_PAGE_X, M_ZERX, M_AC,   4, 0}, /* Zeropage,X */
    /* 36 */ {  "rol",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0},/* Zeropage,X */
    /* 37 */ {  "rla",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0},

    /* 38 */ {  "sec",  IMPLIED, M_NONE, M_FC, 2, 0},
    /* 39 */ {  "and",  ABSOLUTE_Y, M_ABSY, M_AC, 4, 1},    /* Absolute,Y */
    /* 3a */ {  "nop",  IMPLIED, M_NONE, M_NONE,  2, 0},
    /* 3b */ {  "rla",  ABSOLUTE_Y, M_ABSY, M_ABSY, 7, 0},

    /* 3c */ {  "nop",  ABSOLUTE_X, M_NONE, M_NONE, 4, 1},
    /* 3d */ {  "and",  ABSOLUTE_X, M_ABSX, M_AC,   4, 1},  /* Absolute,X */
    /* 3e */ {  "rol",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0}, /* Absolute,X */
    /* 3f */ {  "rla",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0},

	/* 40 */ {	"rti",	IMPLIED, M_NONE, M_PC, 6, 0},
    /* 41 */ {  "eor",  INDIRECT_X, M_INDX, M_AC, 6, 0},    /* (Indirect,X) */
    /* 42 */ {  "jam",  IMPLIED, M_NONE, M_NONE, 0, 0}, /* TILT */
    /* 43 */ {  "sre",  INDIRECT_X, M_INDX, M_INDX, 8, 0},

    /* 44 */ {  "nop",  ZERO_PAGE, M_NONE, M_NONE, 3, 0},
    /* 45 */ {  "eor",  ZERO_PAGE, M_ZERO, M_AC,   3, 0},   /* Zeropage */
    /* 46 */ {  "lsr",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},   /* Zeropage */
    /* 47 */ {  "sre",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},

	/* 48 */ {	"pha",	IMPLIED, M_AC, M_NONE,   3, 0},
    /* 49 */ {  "eor",  IMMEDIATE, M_IMM, M_AC,  2, 0}, /* Immediate */
    /* 4a */ {  "lsr",  ACCUMULATOR, M_AC, M_AC, 2, 0}, /* Accumulator */
    /* 4b */ {  "asr",  IMMEDIATE, M_ACIM, M_AC, 2, 0}, /* (AC & IMM) >>1 */

    /* 4c */ {  "jmp",  ABSOLUTE, M_ADDR, M_PC, 3, 0},  /* Absolute */
    /* 4d */ {  "eor",  ABSOLUTE, M_ABS, M_AC,  4, 0},  /* Absolute */
    /* 4e */ {  "lsr",  ABSOLUTE, M_ABS, M_ABS, 6, 0},  /* Absolute */
    /* 4f */ {  "sre",  ABSOLUTE, M_ABS, M_ABS, 6, 0},

	/* 50 */ {	"bvc",	RELATIVE, M_REL, M_NONE,  2, 0},
    /* 51 */ {  "eor",  INDIRECT_Y, M_INDY, M_AC, 5, 1}, /* (Indirect),Y */
    /* 52 */ {  "jam",  IMPLIED, M_NONE, M_NONE,  0, 0},    /* TILT */
    /* 53 */ {  "sre",  INDIRECT_Y, M_INDY, M_INDY, 8, 0},

    /* 54 */ {  "nop",  ZERO_PAGE_X, M_NONE, M_NONE, 4, 0},
    /* 55 */ {  "eor",  ZERO_PAGE_X, M_ZERX, M_AC,   4, 0}, /* Zeropage,X */
    /* 56 */ {  "lsr",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0},/* Zeropage,X */
    /* 57 */ {  "sre",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0},

	/* 58 */ {	"cli",	IMPLIED, M_NONE, M_FI,     2, 0},
    /* 59 */ {  "eor",  ABSOLUTE_Y, M_ABSY, M_AC,  4, 1},   /* Absolute,Y */
    /* 5a */ {  "nop",  IMPLIED, M_NONE, M_NONE,   2, 0},
    /* 5b */ {  "sre",  ABSOLUTE_Y, M_ABSY, M_ABSY, 7, 0},

    /* 5c */ {  "nop",  ABSOLUTE_X, M_NONE, M_NONE, 4, 1},
    /* 5d */ {  "eor",  ABSOLUTE_X, M_ABSX, M_AC,   4, 1},  /* Absolute,X */
    /* 5e */ {  "lsr",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0},  /* Absolute,X */
    /* 5f */ {  "sre",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0},

    /* 60 */ {  "rts",  IMPLIED, M_NONE, M_PC, 6, 0},
    /* 61 */ {  "adc",  INDIRECT_X, M_INDX, M_AC, 6, 0},    /* (Indirect,X) */
    /* 62 */ {  "jam",  IMPLIED, M_NONE, M_NONE, 0, 0}, /* TILT */
    /* 63 */ {  "rra",  INDIRECT_X, M_INDX, M_INDX, 8, 0},

    /* 64 */ {  "nop",  ZERO_PAGE, M_NONE, M_NONE, 3, 0},
    /* 65 */ {  "adc",  ZERO_PAGE, M_ZERO, M_AC,   3, 0},   /* Zeropage */
    /* 66 */ {  "ror",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},   /* Zeropage */
    /* 67 */ {  "rra",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},

	/* 68 */ {	"pla",	IMPLIED, M_NONE, M_AC,   4, 0},
    /* 69 */ {  "adc",  IMMEDIATE, M_IMM, M_AC,  2, 0}, /* Immediate */
    /* 6a */ {  "ror",  ACCUMULATOR, M_AC, M_AC, 2, 0}, /* Accumulator */
    /* 6b */ {  "arr",  IMMEDIATE, M_ACIM, M_AC, 2, 0}, /* arr isn't typo */

    /* 6c */ {  "jmp",  ABS_INDIRECT, M_AIND, M_PC,  5, 0}, /* Indirect */
    /* 6d */ {  "adc",  ABSOLUTE, M_ABS, M_AC,  4, 0},  /* Absolute */
    /* 6e */ {  "ror",  ABSOLUTE, M_ABS, M_ABS, 6, 0},  /* Absolute */
    /* 6f */ {  "rra",  ABSOLUTE, M_ABS, M_ABS, 6, 0},

	/* 70 */ {	"bvs",	RELATIVE, M_REL, M_NONE,  2, 0},
    /* 71 */ {  "adc",  INDIRECT_Y, M_INDY, M_AC, 5, 1}, /* (Indirect),Y */
    /* 72 */ {  "jam",  IMPLIED, M_NONE, M_NONE,  0, 0}, /* TILT relative? */
    /* 73 */ {  "rra",  INDIRECT_Y, M_INDY, M_INDY, 8, 0},

    /* 74 */ {  "nop",  ZERO_PAGE_X, M_NONE, M_NONE, 4, 0},
    /* 75 */ {  "adc",  ZERO_PAGE_X, M_ZERX, M_AC,   4, 0}, /* Zeropage,X */
    /* 76 */ {  "ror",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0}, /* Zeropage,X */
    /* 77 */ {  "rra",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0},

	/* 78 */ {	"sei",	IMPLIED, M_NONE, M_FI, 2, 0},
    /* 79 */ {  "adc",  ABSOLUTE_Y, M_ABSY, M_AC, 4, 1},    /* Absolute,Y */
    /* 7a */ {  "nop",  IMPLIED, M_NONE, M_NONE,  2, 0},
    /* 7b */ {  "rra",  ABSOLUTE_Y, M_ABSY, M_ABSY, 7, 0},

    /* 7c */ {  "nop",  ABSOLUTE_X, M_NONE, M_NONE, 4, 1},
    /* 7d */ {  "adc",  ABSOLUTE_X, M_ABSX, M_AC,   4, 1},  /* Absolute,X */
    /* 7e */ {  "ror",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0},  /* Absolute,X */
    /* 7f */ {  "rra",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0},

	/****  Negative  ****/

    /* 80 */ {  "nop",  IMMEDIATE, M_NONE, M_NONE, 2, 0},
    /* 81 */ {  "sta",  INDIRECT_X, M_AC, M_INDX,  6, 0},   /* (Indirect,X) */
    /* 82 */ {  "nop",  IMMEDIATE, M_NONE, M_NONE,  2, 0},
    /* 83 */ {  "sax",  INDIRECT_X, M_ANXR, M_INDX, 6, 0},

    /* 84 */ {  "sty",  ZERO_PAGE, M_YR, M_ZERO,  3, 0},    /* Zeropage */
    /* 85 */ {  "sta",  ZERO_PAGE, M_AC, M_ZERO,  3, 0},    /* Zeropage */
    /* 86 */ {  "stx",  ZERO_PAGE, M_XR, M_ZERO,  3, 0},    /* Zeropage */
    /* 87 */ {  "sax",  ZERO_PAGE, M_ANXR, M_ZERO, 3, 0},

	/* 88 */ {	"dey",	IMPLIED, M_YR, M_YR, 2, 0},
    /* 89 */ {  "nop",  IMMEDIATE, M_NONE, M_NONE, 2, 0},
	/* 8a */ {	"txa",	IMPLIED, M_XR, M_AC, 2, 0},
	/****  very abnormal: usually AC = AC | #$EE & XR & #$oper  ****/
    /* 8b */ {  "ane",  IMMEDIATE, M_AXIM, M_AC, 2, 0},

    /* 8c */ {  "sty",  ABSOLUTE, M_YR, M_ABS, 4, 0},   /* Absolute */
    /* 8d */ {  "sta",  ABSOLUTE, M_AC, M_ABS, 4, 0},   /* Absolute */
    /* 8e */ {  "stx",  ABSOLUTE, M_XR, M_ABS, 4, 0},   /* Absolute */
    /* 8f */ {  "sax",  ABSOLUTE, M_ANXR, M_ABS, 4, 0},

	/* 90 */ {	"bcc",	RELATIVE, M_REL, M_NONE, 2, 0},
    /* 91 */ {  "sta",  INDIRECT_Y, M_AC, M_INDY, 6, 0},    /* (Indirect),Y */
    /* 92 */ {  "jam",  IMPLIED, M_NONE, M_NONE, 0, 0}, /* TILT relative? */
    /* 93 */ {  "sha",  INDIRECT_Y, M_ANXR, M_STH0, 6, 0},

    /* 94 */ {  "sty",  ZERO_PAGE_X, M_YR, M_ZERX, 4, 0},   /* Zeropage,X */
    /* 95 */ {  "sta",  ZERO_PAGE_X, M_AC, M_ZERX, 4, 0},   /* Zeropage,X */
    /* 96 */ {  "stx",  ZERO_PAGE_Y, M_XR, M_ZERY, 4, 0},   /* Zeropage,Y */
    /* 97 */ {  "sax",  ZERO_PAGE_Y, M_ANXR, M_ZERY, 4, 0},

	/* 98 */ {	"tya",	IMPLIED, M_YR, M_AC, 2, 0},
    /* 99 */ {  "sta",  ABSOLUTE_Y, M_AC, M_ABSY, 5, 0},    /* Absolute,Y */
	/* 9a */ {	"txs",	IMPLIED, M_XR, M_SP, 2, 0},
    /*** This is very mysterious comm and ... */
    /* 9b */ {  "shs",  ABSOLUTE_Y, M_ANXR, M_STH3, 5, 0},

    /* 9c */ {  "shy",  ABSOLUTE_X, M_YR, M_STH2, 5, 0},
    /* 9d */ {  "sta",  ABSOLUTE_X, M_AC, M_ABSX, 5, 0},    /* Absolute,X */
    /* 9e */ {  "shx",  ABSOLUTE_Y, M_XR, M_STH1, 5, 0},
    /* 9f */ {  "sha",  ABSOLUTE_Y, M_ANXR, M_STH1, 5, 0},

    /* a0 */ {  "ldy",  IMMEDIATE, M_IMM, M_YR, 2, 0},  /* Immediate */
    /* a1 */ {  "lda",  INDIRECT_X, M_INDX, M_AC, 6, 0},    /* (indirect,X) */
    /* a2 */ {  "ldx",  IMMEDIATE, M_IMM, M_XR, 2, 0},  /* Immediate */
    /* a3 */ {  "lax",  INDIRECT_X, M_INDX, M_ACXR, 6, 0},  /* (indirect,X) */

    /* a4 */ {  "ldy",  ZERO_PAGE, M_ZERO, M_YR, 3, 0}, /* Zeropage */
    /* a5 */ {  "lda",  ZERO_PAGE, M_ZERO, M_AC, 3, 0}, /* Zeropage */
    /* a6 */ {  "ldx",  ZERO_PAGE, M_ZERO, M_XR, 3, 0}, /* Zeropage */
    /* a7 */ {  "lax",  ZERO_PAGE, M_ZERO, M_ACXR, 3, 0},

	/* a8 */ {	"tay",	IMPLIED, M_AC, M_YR,    2, 0},
    /* a9 */ {  "lda",  IMMEDIATE, M_IMM, M_AC, 2, 0},  /* Immediate */
    /* aa */ {  "tax",  IMPLIED, M_AC, M_XR,    2, 0},
    /* ab */ {  "lxa",  IMMEDIATE, M_ACIM, M_ACXR, 2, 0},   /* LXA isn't a typo */

    /* ac */ {  "ldy",  ABSOLUTE, M_ABS, M_YR, 4, 0},   /* Absolute */
    /* ad */ {  "lda",  ABSOLUTE, M_ABS, M_AC, 4, 0},   /* Absolute */
    /* ae */ {  "ldx",  ABSOLUTE, M_ABS, M_XR, 4, 0},   /* Absolute */
    /* af */ {  "lax",  ABSOLUTE, M_ABS, M_ACXR, 4, 0},

	/* b0 */ { 	"bcs",	RELATIVE, M_REL, M_NONE,  2, 0},
    /* b1 */ {  "lda",  INDIRECT_Y, M_INDY, M_AC, 5, 1}, /* (indirect),Y */
    /* b2 */ {  "jam",  IMPLIED, M_NONE, M_NONE,  0, 0}, /* TILT */
    /* b3 */ {  "lax",  INDIRECT_Y, M_INDY, M_ACXR, 5, 1},

    /* b4 */ {  "ldy",  ZERO_PAGE_X, M_ZERX, M_YR, 4, 0},   /* Zeropage,X */
    /* b5 */ {  "lda",  ZERO_PAGE_X, M_ZERX, M_AC, 4, 0},   /* Zeropage,X */
    /* b6 */ {  "ldx",  ZERO_PAGE_Y, M_ZERY, M_XR, 4, 0},   /* Zeropage,Y */
    /* b7 */ {  "lax",  ZERO_PAGE_Y, M_ZERY, M_ACXR, 4, 0},

	/* b8 */ {	"clv",	IMPLIED, M_NONE, M_FV,    2, 0},
    /* b9 */ {  "lda",  ABSOLUTE_Y, M_ABSY, M_AC, 4, 1},    /* Absolute,Y */
	/* ba */ {	"txs",	IMPLIED, M_SP, M_XR,      2, 0},
    /* bb */ {  "las",  ABSOLUTE_Y, M_SABY, M_ACXS, 4, 1},

    /* bc */ {  "ldy",  ABSOLUTE_X, M_ABSX, M_YR, 4, 1},    /* Absolute,X */
    /* bd */ {  "lda",  ABSOLUTE_X, M_ABSX, M_AC, 4, 1},    /* Absolute,X */
    /* be */ {  "ldx",  ABSOLUTE_Y, M_ABSY, M_XR, 4, 1},    /* Absolute,Y */
    /* bf */ {  "lax",  ABSOLUTE_Y, M_ABSY, M_ACXR, 4, 1},

    /* c0 */ {  "cpy",  IMMEDIATE, M_IMM, M_NONE, 2, 0},    /* Immediate */
    /* c1 */ {  "cmp",  INDIRECT_X, M_INDX, M_NONE, 6, 0},  /* (Indirect,X) */
    /* c2 */ {  "nop",  IMMEDIATE, M_NONE, M_NONE, 2, 0},   /* occasional TILT */
    /* c3 */ {  "dcp",  INDIRECT_X, M_INDX, M_INDX, 8, 0},

    /* c4 */ {  "cpy",  ZERO_PAGE, M_ZERO, M_NONE, 3, 0},   /* Zeropage */
    /* c5 */ {  "cmp",  ZERO_PAGE, M_ZERO, M_NONE, 3, 0},   /* Zeropage */
    /* c6 */ {  "dec",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},   /* Zeropage */
    /* c7 */ {  "dcp",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},

    /* c8 */ {  "iny",  IMPLIED, M_YR, M_YR, 2, 0},
    /* c9 */ {  "cmp",  IMMEDIATE, M_IMM, M_NONE, 2, 0},    /* Immediate */
	/* ca */ {	"dex",	IMPLIED, M_XR, M_XR, 2, 0},
    /* cb */ {  "sbx",  IMMEDIATE, M_IMM, M_XR, 2, 0},

    /* cc */ {  "cpy",  ABSOLUTE, M_ABS, M_NONE, 4, 0}, /* Absolute */
    /* cd */ {  "cmp",  ABSOLUTE, M_ABS, M_NONE, 4, 0}, /* Absolute */
    /* ce */ {  "dec",  ABSOLUTE, M_ABS, M_ABS,  6, 0}, /* Absolute */
    /* cf */ {  "dcp",  ABSOLUTE, M_ABS, M_ABS,  6, 0},

	/* d0 */ {	"bne",	RELATIVE, M_REL, M_NONE, 2, 0},
    /* d1 */ {  "cmp",  INDIRECT_Y, M_INDY, M_NONE, 5, 1},  /* (Indirect),Y */
    /* d2 */ {  "jam",  IMPLIED, M_NONE, M_NONE,    0, 0},  /* TILT */
    /* d3 */ {  "dcp",  INDIRECT_Y, M_INDY, M_INDY, 8, 0},

    /* d4 */ {  "nop",  ZERO_PAGE_X, M_NONE, M_NONE, 4, 0},
    /* d5 */ {  "cmp",  ZERO_PAGE_X, M_ZERX, M_NONE, 4, 0},/* Zeropage,X */
    /* d6 */ {  "dec",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0},/* Zeropage,X */
    /* d7 */ {  "dcp",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0},

	/* d8 */ {	"cld",	IMPLIED, M_NONE, M_FD, 2, 0},
    /* d9 */ {  "cmp",  ABSOLUTE_Y, M_ABSY, M_NONE, 4, 1},  /* Absolute,Y */
    /* da */ {  "nop",  IMPLIED, M_NONE, M_NONE,    2, 0},
    /* db */ {  "dcp",  ABSOLUTE_Y, M_ABSY, M_ABSY, 7, 0},

    /* dc */ {  "nop",  ABSOLUTE_X, M_NONE, M_NONE, 4, 1},
    /* dd */ {  "cmp",  ABSOLUTE_X, M_ABSX, M_NONE, 4, 1},  /* Absolute,X */
    /* de */ {  "dec",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0},  /* Absolute,X */
    /* df */ {  "dcp",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0},

    /* e0 */ {  "cpx",  IMMEDIATE, M_IMM, M_NONE, 2, 0},    /* Immediate */
    /* e1 */ {  "sbc",  INDIRECT_X, M_INDX, M_AC, 6, 0},    /* (Indirect,X) */
    /* e2 */ {  "nop",  IMMEDIATE, M_NONE, M_NONE,  2, 0},
    /* e3 */ {  "isb",  INDIRECT_X, M_INDX, M_INDX, 8, 0},

    /* e4 */ {  "cpx",  ZERO_PAGE, M_ZERO, M_NONE, 3, 0},   /* Zeropage */
    /* e5 */ {  "sbc",  ZERO_PAGE, M_ZERO, M_AC,   3, 0},   /* Zeropage */
    /* e6 */ {  "inc",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},   /* Zeropage */
    /* e7 */ {  "isb",  ZERO_PAGE, M_ZERO, M_ZERO, 5, 0},

    /* e8 */ {  "inx",  IMPLIED, M_XR, M_XR,     2, 0},
    /* e9 */ {  "sbc",  IMMEDIATE, M_IMM, M_AC,  2, 0}, /* Immediate */
    /* ea */ {  "nop",  IMPLIED, M_NONE, M_NONE, 2, 0},
    /* eb */ {  "usbc", IMMEDIATE, M_IMM, M_AC,  2, 0}, /* same as e9 */

    /* ec */ {  "cpx",  ABSOLUTE, M_ABS, M_NONE, 4, 0}, /* Absolute */
    /* ed */ {  "sbc",  ABSOLUTE, M_ABS, M_AC,  4, 0},  /* Absolute */
    /* ee */ {  "inc",  ABSOLUTE, M_ABS, M_ABS, 6, 0},  /* Absolute */
    /* ef */ {  "isb",  ABSOLUTE, M_ABS, M_ABS, 6, 0},

	/* f0 */ {	"beq",	RELATIVE, M_REL, M_NONE,  2, 0},
    /* f1 */ {  "sbc",  INDIRECT_Y, M_INDY, M_AC, 5, 1}, /* (Indirect),Y */
    /* f2 */ {  "jam",  IMPLIED, M_NONE, M_NONE,  0, 0}, /* TILT */
    /* f3 */ {  "isb",  INDIRECT_Y, M_INDY, M_INDY, 8, 0},

    /* f4 */ {  "nop",  ZERO_PAGE_X, M_NONE, M_NONE, 4, 0},
    /* f5 */ {  "sbc",  ZERO_PAGE_X, M_ZERX, M_AC,   4, 0}, /* Zeropage,X */
    /* f6 */ {  "inc",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0}, /* Zeropage,X */
    /* f7 */ {  "isb",  ZERO_PAGE_X, M_ZERX, M_ZERX, 6, 0},

	/* f8 */ {	"sed",	IMPLIED, M_NONE, M_FD,    2, 0},
    /* f9 */ {  "sbc",  ABSOLUTE_Y, M_ABSY, M_AC, 4, 1}, /* Absolute,Y */
    /* fa */ {  "nop",  IMPLIED, M_NONE, M_NONE,  2, 0},
    /* fb */ {  "isb",  ABSOLUTE_Y, M_ABSY, M_ABSY, 7, 0},

    /* fc */ {  "nop",  ABSOLUTE_X, M_NONE, M_NONE, 4, 1},
    /* fd */ {  "sbc",  ABSOLUTE_X, M_ABSX, M_AC,   4, 1},  /* Absolute,X */
    /* fe */ {  "inc",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0},  /* Absolute,X */
    /* ff */ {  "isb",  ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0}
};
